from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

__all__ = [
    "__title__", "__summary__", "__uri__", "__version__", "__author__",
    "__email__", "__license__", "__copyright__",
]

__title__ = "recliner"
__summary__ = ""
__uri__ = "https://github.com/crateio/recliner/"

__version__ = "0.2.1"

__author__ = "Donald Stufft"
__email__ = "donald.stufft@gmail.com"

__license__ = "Simplified BSD"
__copyright__ = "Copyright 2012 Donald Stufft"
